#include <PinChangeInt.h>

int motorPin = 6; // Controle de la vitese du moteur
int rotSensorPin = 8; // signal du compte compte-tour

int distSensorPin1 = 11; // Signal du capteur Sick
int distSensorPin2 = 12; //Utilisation de 2 pins pour gèrer les interruptions

//Variables gloables pour les calculs

volatile double rotSpeed = 0.0;
volatile long timeSpeed = 0;
volatile double deltaSpeed = 0.0;
volatile double deltaSpeedLisse = 0.0;

volatile double distTimeTop = 0.0;
volatile double distTime = 0.0;

volatile double angleTop = 0.0;
volatile double angleStop = 0.0;


volatile long timeTop = 0;
volatile long timeStop = 0;

volatile int testCount=0;


void setup()
{
  pinMode(motorPin, OUTPUT); // définitions des Entrées/Sorties
  pinMode(rotSensorPin, INPUT);
  pinMode(distSensorPin1, INPUT);
  pinMode(distSensorPin2, INPUT);
  
  attachPinChangeInterrupt(rotSensorPin, updateSpeed, FALLING); // Mise à jour de la vitesse sur front descendant
  attachPinChangeInterrupt(distSensorPin1, contactDistTop, RISING); // Activation du chrono sur front montant
  attachPinChangeInterrupt(distSensorPin2, contactDistStop, FALLING); // Désativation du chrono sur front descendant
  
  Serial.begin(9600); // ouverture de la connexion série
}

void loop()// boucle principale
{
  analogWrite(motorPin, 55); // Définition de la vitesse du moteur
  delay(250);
}

void updateSpeed() // mise à jour de la vitesse
{
  deltaSpeed = (micros() - timeSpeed);
  timeSpeed = micros();
  rotSpeed = (1000.0*1000.0*60.0)/deltaSpeed;// calcul de la vitesse
  sendReset();
}


void contactDistTop() // début de contact avec la balise
{
    timeTop=micros();
}

void contactDistStop() // fin de contact avec la balise
{
   timeStop=micros();
 
   angleTop= (timeTop-timeSpeed)/deltaSpeed; // calculs des angles de début et de fin de contact avec la balise
   angleStop= (timeStop-timeSpeed)/deltaSpeed;
   
   int angle=((timeTop+timeStop)/2-timeSpeed)/(deltaSpeed)*360;
   if(angle<0){angle+=360;} // on veut un angle positif
   int dist=4/(abs(angleStop-angleTop)*2.0*3.14); // Attention r=4cm donc la distance est donnée en cm
   
   sendData(angle,dist);// envoi des données sur la liaison série
}

//void refreshAff2() // fonction d'affichage pour debug sur le moniteur série Arduino sur PC
//{
// Serial.print(rotSpeed);
// Serial.print(",");
// Serial.print((angleStop-angleTop)*360);
// Serial.print("   ");
// Serial.print(0);
// Serial.println(",");
// //Serial.print("  distance brute : ");
// //Serial.println(dist);
//}

void sendData(int angle,int distance) // fonction d'envoi des données sur la liaison série
{
  byte val[5]={250,3,60,200,255}; // balise de début : 250, balise de fin : 255
  // la valeur de l'angle supérieure à 255, il faut donc 2 bytes pour la coder
  val[1]= angle/256;// première valeur : centaine de l'angle
  val[2]= angle%256;
  val[3]= distance;
  int len = 5;
  Serial.write(val,len);
  //refreshAff2();
  
}
  //Serial.println(distance);
void sendReset()
{
  Serial.write(254); //envoi du drapeau 254 sur la liaison série pour stipuler la fin d'un tour
}
